// Author: George W. Taylor
// http://www.tropicalcoder.com/
// Date: 15 Oct 2012
// License: Creative Commons - Non-Commercial Use

#include "stdafx.h"
#include "PortMonitor.h"
#include "Debug.h"

PortMonitor::PortMonitor()
{
 hCom = NULL;
 ZeroMemory(&o, sizeof(OVERLAPPED));
}

BOOL PortMonitor::InitPort(HWND hwnd, char *PortName, DWORD dwComEvent)
{
 char Msg[64];
 bPostedErrorMsg = false;

 this->dwComEvent = dwComEvent;

 // ConfigurePort(NULL, hwnd, PortName); // ??? Do we need this for something?

 hCom = CreateFile(PortName,
        GENERIC_READ | GENERIC_WRITE,
        0,    // exclusive access 
        NULL, // default security attributes 
        OPEN_EXISTING,
        FILE_FLAG_OVERLAPPED,
        NULL 
        );

  if(hCom == INVALID_HANDLE_VALUE)
  {
	 strcpy(Msg, "Couldn't open port ");
	 strcat(Msg, PortName);
     MessageBox(hwnd, Msg, "Error", MB_OK | MB_ICONHAND);
     return FALSE;
  }

  // Set the event mask. 
  BOOL fSuccess = SetCommMask(hCom, dwComEvent);

  if(!fSuccess) 
  {
     // Handle the error. 
     MessageBox(hwnd, "SetCommMask failed", "Error", MB_OK | MB_ICONHAND);
     return FALSE;
  }

  // Create an event object for use by WaitCommEvent. 
  o.hEvent = CreateEvent(
        NULL,   // default security attributes 
        TRUE,   // manual-reset event 
        FALSE,  // not signaled 
        NULL    // no name
		);
 return TRUE;
}


int PortMonitor::ReadPort(void)
{
 DWORD dwEvtMask;

 // Initialize the rest of the OVERLAPPED structure to zero.
 o.Internal = 0;
 o.InternalHigh = 0;
 o.Offset = 0;
 o.OffsetHigh = 0;

 if(WaitCommEvent(hCom, &dwEvtMask, &o)) 
 {
    // The signal being monitored changed state.
    return 1;
 }

 // IO pending?
 if(ERROR_IO_PENDING == GetLastError())
 {
	DWORD dwIrrelevant;

    // Check on an asynchronous read operation.
	// function does not return until the operation has been completed
    if(GetOverlappedResult(hCom, &o, &dwIrrelevant, TRUE))
	{
       // The signal being monitored changed state.
       return 1;
	}
	// The com event was cleared (as is done on shut down)
	return 0;
 }

 // There was an error
 return -1;
}


bool PortMonitor::LineSignaled(void)
{
 DWORD dwModemStat = 0;

 if(!GetCommModemStatus(hCom, &dwModemStat))
 {
	 if(!bPostedErrorMsg)
	 {
	    DebugMessage("Failed: GetCommModemStatus!");
		bPostedErrorMsg = true;
	 }
	return false;
 }

 if(dwModemStat == 0)return false;

 if((dwComEvent == EV_CTS)  && (dwModemStat & MS_CTS_ON))return true;
 if((dwComEvent == EV_DSR)  && (dwModemStat & MS_DSR_ON))return true;
 if((dwComEvent == EV_RING) && (dwModemStat & MS_RING_ON))return true;
 if((dwComEvent == EV_RLSD) && (dwModemStat & MS_RLSD_ON))return true;

 return false;
}


void PortMonitor::CancelComWait(void)
{
 if((hCom != NULL) && (hCom != INVALID_HANDLE_VALUE))
 {
	 CancelIoEx(hCom, &o);
     if(o.hEvent != NULL)
     {
	    ResetEvent(o.hEvent);
     }
 }
}
